@@ -1,24 +1,38 @@
 use core::fmt;
 use std::cell::RefCell;
+use std::collections::HashSet;
 
 use proc_macro2::Span;
 use quote::ToTokens;
 
+struct Inner {
+    modes: HashSet<syn::ExprPath>,
+    errors: Vec<syn::Error>,
+}
+
 pub(crate) struct Ctxt {
-    errors: RefCell<Vec<syn::Error>>,
+    inner: RefCell<Inner>,
 }
 
 impl Ctxt {
     /// Construct a new handling context.
     pub(crate) fn new() -> Self {
         Self {
-            errors: RefCell::new(Vec::new()),
+            inner: RefCell::new(Inner {
+                modes: HashSet::new(),
+                errors: Vec::new(),
+            }),
         }
     }
 
+    /// Register a new mode.
+    pub(crate) fn register_mode(&self, mode: syn::ExprPath) {
+        self.inner.borrow_mut().modes.insert(mode);
+    }
+
     /// Test if context contains errors.
     pub(crate) fn has_errors(&self) -> bool {
-        !self.errors.borrow().is_empty()
+        !self.inner.borrow().errors.is_empty()
     }
 
     /// Report an error with a span.
@@ -27,8 +41,9 @@ impl Ctxt {
         S: ToTokens,
         T: fmt::Display,
     {
-        self.errors
+        self.inner
             .borrow_mut()
+            .errors
             .push(syn::Error::new_spanned(spanned, message));
     }
 
@@ -37,18 +52,24 @@ impl Ctxt {
     where
         T: fmt::Display,
     {
-        self.errors
+        self.inner
             .borrow_mut()
+            .errors
             .push(syn::Error::new(span, message));
     }
 
     /// Error reported directly by syn.
     pub(crate) fn syn_error(&self, error: syn::Error) {
-        self.errors.borrow_mut().push(error);
+        self.inner.borrow_mut().errors.push(error);
     }
 
     /// Access interior errors.
     pub(crate) fn into_errors(self) -> Vec<syn::Error> {
-        std::mem::take(&mut *self.errors.borrow_mut())
+        std::mem::take(&mut self.inner.borrow_mut().errors)
+    }
+
+    /// Get all "extra" modes specified.
+    pub(crate) fn modes(&self) -> Vec<syn::ExprPath> {
+        self.inner.borrow().modes.iter().cloned().collect()
     }
 }
